@@ -9,22 +9,124 @@ from org.eclipse.swtbot.swt.finder.exceptions import WidgetNotFoundException
 from org.eclipse.jface.viewers import ISelectionChangedListener
 from org.eclipse.ui.internal import EditorReference
 # Force classloading in the test thread where it works...
+from org.eclipse.draw2d.geometry import Insets #@UnusedWildImport
 from org.eclipse.draw2d import * #@UnusedWildImport
 from org.eclipse.gef import *
 import storytext.guishared
 from org.eclipse import swt
-import time
+import time, logging
 
 class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
+    widgetMonitor = None
     def __init__(self, botOrGefViewer):
         gefViewer = self._getViewer(botOrGefViewer) if isinstance(botOrGefViewer, gefbot.widgets.SWTBotGefViewer) else botOrGefViewer
         gefbot.widgets.SWTBotGefViewer.__init__(self, gefViewer)
+        self.logger = logging.getLogger("Centre finding")
+
+    def findOverlap(self, overlaps, centre):
+        for overlap in overlaps:
+            if overlap.contains(centre):
+                return overlap
+
+    def getCenter(self, editPart):
+        bounds = self.getBoundsInternal(editPart)
+        self.logger.debug("Object bounds at " + repr(bounds))
+        viewportBounds = self.getFigureCanvas().getViewportBounds()
+        self.logger.debug("Canvas viewport bounds at " + repr(viewportBounds))
+        visibleBounds = viewportBounds.getIntersection(bounds)
+        if getInt(visibleBounds.height) == 0 or getInt(visibleBounds.width) == 0:
+            self.logger.debug("No intersection with viewport, scrolling until there is")
+            self.ensureInViewport(bounds, viewportBounds)
+            visibleBounds = self.getBoundsInternal(editPart)
+
+        self.logger.debug("Found bounds at " + repr(visibleBounds))
+        overlaps = self.findOverlapRegions(editPart, visibleBounds)
+        return self.findNonOverlappingCentre(visibleBounds, overlaps)
+
+    def getEdges(self, bounds):
+        topEdge = getInt(bounds.y)
+        bottomEdge = topEdge + getInt(bounds.height)
+        leftEdge = getInt(bounds.x)
+        rightEdge = leftEdge + getInt(bounds.width)
+        return topEdge, leftEdge, bottomEdge, rightEdge
+
+    def ensureInViewport(self, bounds, viewportBounds):
+        viewTop, viewLeft, viewBottom, viewRight = self.getEdges(viewportBounds)
+        top, left, bottom, right = self.getEdges(bounds)
+        canvas = self.getFigureCanvas()
+        if bottom <= viewTop:
+            canvas.scrollYOffset(top - viewTop)
+        elif top >= viewBottom:
+            canvas.scrollYOffset(bottom - viewBottom)
+        if right <= viewLeft:
+            canvas.scrollXOffset(left - viewLeft)
+        elif left >= viewRight:
+            canvas.scrollXOffset(right - viewRight)
+            
+    def findNonOverlappingCentre(self, bounds, overlaps):
+        centre = bounds.getCenter()
+        self.logger.debug("Found centre at " + repr(centre))
+        overlap = self.findOverlap(overlaps, centre)
+        if overlap:
+            self.logger.debug("Centre overlaps rectangle at " + repr(overlap))
+            for rect in self.findOutsideRectangles(bounds, overlap):
+                self.logger.debug("Trying outside rectangle " + repr(rect))
+                newOverlaps = []
+                for otherOverlap in overlaps:
+                    if otherOverlap is not overlap:
+                        intersection = rect.getIntersection(otherOverlap)
+                        if getInt(intersection.height) and getInt(intersection.width) and intersection not in newOverlaps:    
+                            self.logger.debug("Found new overlap " + repr(intersection))
+                            newOverlaps.append(intersection)
+                if rect not in newOverlaps:
+                    centre = self.findNonOverlappingCentre(rect, newOverlaps)
+                    if centre:
+                        return centre
+        return getInt(centre.x), getInt(centre.y)
+    
+    def findOutsideRectangles(self, rect, innerRect):
+        top, left, bottom, right = self.getEdges(rect)
+        innertop, innerleft, innerbottom, innerright = self.getEdges(innerRect)
+        insets = [ Insets(innerbottom - top, 0, 0, 0), 
+                   Insets(0, innerright - left, 0, 0),
+                   Insets(0, 0, bottom - innertop, 0),
+                   Insets(0, 0, 0, right - innerleft) ]
+        rects = [ rect.getCropped(i) for i in insets ]
+        rects = filter(lambda r: getInt(r.height) and getInt(r.width), rects)
+        return sorted(rects, key=lambda r: -getInt(r.height) * getInt(r.width))
+            
+    def findRoot(self, editPart):
+        parent = editPart.parent()
+        if parent.part() is None:
+            return editPart
+        else:
+            return self.findRoot(parent)
+
+    def findOverlapRegions(self, editPart, bounds):
+        rootEditPart = self.findRoot(editPart)
+        return self.findOverlapRegionsUnder(rootEditPart, editPart, bounds)
+    
+    def findOverlapRegionsUnder(self, editPart, ignoreEditPart, bounds):
+        return self.findOverlapRegionsFor(editPart.children(), ignoreEditPart, bounds) + \
+            self.findOverlapRegionsFor(editPart.sourceConnections(), ignoreEditPart, bounds)
+    
+    def findOverlapRegionsFor(self, editParts, ignoreEditPart, bounds):
+        overlaps = []
+        for otherPart in editParts:
+            if otherPart is not ignoreEditPart and otherPart.part().isSelectable():
+                otherBounds = self.getBoundsInternal(otherPart)
+                intersection = bounds.getIntersection(otherBounds)
+                if getInt(intersection.height) and getInt(intersection.width) and intersection != bounds and intersection not in overlaps:
+                    overlaps.append(intersection)
+                    self.logger.debug("Overlap found at " + repr(intersection))
+            overlaps += self.findOverlapRegionsUnder(otherPart, ignoreEditPart, bounds)
+        return overlaps
 
     def clickOnCenter(self, editPart, keyModifiers=0):
-        bounds = self.getBoundsInternal(editPart)
-        visibleBounds = self.getFigureCanvas().getIntersection(bounds)
-        centreX = visibleBounds.x + visibleBounds.width / 2
-        centreY = visibleBounds.y + visibleBounds.height / 2
+        if self.widgetMonitor:
+            # To do actual clicking, must make sure the shell is active... doesn't happen automatically in Xvfb
+            self.widgetMonitor.forceShellActive()
+        centreX, centreY = self.getCenter(editPart)
         # x and y should be public fields, and are sometimes. In our tests, they are methods, for some unknown reason
         self.getFigureCanvas().mouseMoveLeftClick(centreX, centreY, keyModifiers)
 
@@ -33,7 +135,7 @@ class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
         declaredMethod = self.getClass().getSuperclass().getDeclaredMethod("getAbsoluteBounds", [gefbot.widgets.SWTBotGefEditPart])
         declaredMethod.setAccessible(True)
         return declaredMethod.invoke(self, [editPart])
-
+        
     def getViewer(self):
         return self._getViewer(self)
     
@@ -54,10 +156,8 @@ class StoryTextSWTBotGefViewer(gefbot.widgets.SWTBotGefViewer):
 
     # toX and toY should be in display coordinates
     def drag(self, editPart, toX, toY, keyModifiers=0):
-        bounds = self.getBoundsInternal(editPart)
-        # Hard coded offset found in swtbot.
-        offset = 7/2 + 1
-        self.getFigureCanvas().mouseDrag(getInt(bounds.x) + offset, getInt(bounds.y) + offset, toX, toY, keyModifiers)
+        centreX, centreY = self.getCenter(editPart)
+        self.getFigureCanvas().mouseDrag(centreX, centreY, toX, toY, keyModifiers)
 
 class StoryTextSWTBotGefFigureCanvas(gefbot.widgets.SWTBotGefFigureCanvas):    
     def mouseDrag(self, fromX, fromY, toX, toY, keyModifiers=0):
@@ -99,13 +199,22 @@ class StoryTextSWTBotGefFigureCanvas(gefbot.widgets.SWTBotGefFigureCanvas):
             startY += counterY
             rcpsimulator.swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.postMouseMove, startX, startY)
             rcpsimulator.swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.waitForCursor, startX, startY)
-
-    def getIntersection(self, bounds):
-        # SWT and draw2d have their own rectangle class, with the same methods, but they aren't the same...
-        swtBounds = swt.graphics.Rectangle(getInt(bounds.x), getInt(bounds.y), getInt(bounds.width), getInt(bounds.height))
-        canvasBounds = rcpsimulator.swtsimulator.runOnUIThread(self.widget.getBounds)
-        return canvasBounds.intersection(swtBounds)
-
+            
+    def getViewportBounds(self):
+        return rcpsimulator.swtsimulator.runOnUIThread(self.widget.getViewport().getBounds)
+    
+    def scrollXOffset(self, offset):
+        def doScroll():
+            currPos = getInt(self.widget.getViewport().getViewLocation().x)
+            self.widget.scrollToX(currPos + offset)
+        rcpsimulator.swtsimulator.runOnUIThread(doScroll)
+        
+    def scrollYOffset(self, offset):
+        def doScroll():
+            currPos = getInt(self.widget.getViewport().getViewLocation().y)
+            self.widget.scrollToY(currPos + offset)
+        rcpsimulator.swtsimulator.runOnUIThread(doScroll)
+        
     def mouseMoveLeftClick(self, x, y, keyModifiers=0):
         displayLoc = rcpsimulator.swtsimulator.runOnUIThread(self.toDisplayLocation, x, y)
         rcpsimulator.swtsimulator.runOnUIThread(storytext.guishared.catchAll, self.postMouseMove, displayLoc.x, displayLoc.y)
@@ -175,13 +284,15 @@ class WidgetMonitor(rcpsimulator.WidgetMonitor):
     def __init__(self, *args, **kw):
         self.allPartRefs = set()
         self.swtbotMap[GraphicalViewer] = (StoryTextSWTBotGefViewer, [])
+        self.logger = logging.getLogger("storytext record")
         rcpsimulator.WidgetMonitor.__init__(self, *args, **kw)
+        StoryTextSWTBotGefViewer.widgetMonitor = self
 
     def createSwtBot(self):
         return gefbot.SWTGefBot()
 
-    def monitorAllWidgets(self, parent, widgets):
-        rcpsimulator.WidgetMonitor.monitorAllWidgets(self, parent, widgets)
+    def monitorAllWidgets(self, widgets):
+        rcpsimulator.WidgetMonitor.monitorAllWidgets(self, widgets)
         self.monitorGefWidgets()
 
     def monitorViewContentsMenus(self, botView):
@@ -190,12 +301,16 @@ class WidgetMonitor(rcpsimulator.WidgetMonitor):
             if menu:
                 self.monitorMenu(menu)
 
+    def sendShowEvent(self, menu):
+        menu.notifyListeners(swt.SWT.Show, swt.widgets.Event())
+        
     def monitorMenu(self, menu):
         if menu.getItemCount() == 0:
             # The menu is a Contribution defined in plugin.xml. We have to send an extra
             # SHOW event to instantiate it.
-            menu.notifyListeners(swt.SWT.Show, swt.widgets.Event())
-        menu.notifyListeners(swt.SWT.Show, swt.widgets.Event())
+            self.sendShowEvent(menu)
+
+        self.sendShowEvent(menu)
         for item in menu.getItems():
             submenu = item.getMenu()
             if submenu:
@@ -267,10 +382,8 @@ class ViewerEvent(storytext.guishared.GuiEvent):
     def outputForScript(self, *args):
         return ' '.join([self.name, self.getStateDescription(*args) ])
 
-    def getStateDescription(self, part, *args):
-        if self.isMainEditPart(part):
-            return self.storeObjectDescription(part)
-        descs = [self.storeObjectDescription(editPart.part()) for editPart in self.widget.selectedEditParts()]
+    def getStateDescription(self, parts, *args):
+        descs = map(self.storeObjectDescription, parts)
         return ','.join(descs)
 
     def addSuffix(self, desc):
@@ -356,25 +469,41 @@ class ViewerSelectEvent(ViewerEvent):
         rcpsimulator.swtsimulator.runOnUIThread(self.getGefViewer().addSelectionChangedListener, SelectionListener())
 
     def applyToSelected(self, event, method):
-        selection = event.getSelection()
-        for editPart in selection.toList():
-            method(editPart, self)
+        method(event.getSelection().toList(), self)
 
-    def generate(self, description, *args):
+    def parseArguments(self, description):
         parts = []
-        for part in description.split(","):
+        notfound = []
+        allParts = description.split(",")
+        for part in allParts:
             editPart = self.findEditPart(self.widget.rootEditPart(), part)
-            if editPart == self.widget.mainEditPart():
-                return self.widget.click(editPart)
             if editPart:
                 parts.append(editPart)
-
+            else:
+                notfound.append(part)
+                                
         if len(parts) > 0:
-            self.widget.clickOnCenter(parts[0])
-            for part in parts[1:]:
-                self.widget.clickOnCenter(part, keyModifiers=swt.SWT.CTRL)
+            if len(notfound) > 0:
+                return self.makePartialParseFailure(parts, ",".join(notfound), allParts[0] not in notfound)
+            else:
+                return parts
         else:
-            raise UseCaseScriptError, "Could not find any objects in viewer matching description " + repr(description)
+            raise UseCaseScriptError, "could not find any objects in viewer matching description "  + repr(description)
+        
+    def generate(self, parts, partial=False):
+        if not partial:
+            rcpsimulator.swtsimulator.runOnUIThread(self.getGefViewer().deselectAll)
+        if len(parts) == 1 and not partial:
+            self.widget.clickOnCenter(parts[0])
+        else:
+            pauseTime = self.getPauseBetweenSelections()
+            for i, part in enumerate(parts):
+                if i > 0 and pauseTime:
+                    time.sleep(pauseTime)
+                self.widget.clickOnCenter(part, swt.SWT.CTRL)
+                
+    def getPauseBetweenSelections(self):
+        return 0 #Don't pause by default, allow customwidgetevents to override this
 
     @classmethod
     def getAssociatedSignal(cls, widget):
@@ -384,14 +513,10 @@ class ViewerSelectEvent(ViewerEvent):
     def getSignalsToFilter(cls):
         return [ swt.SWT.MouseDown, swt.SWT.MouseUp ]
 
-    def shouldRecord(self, part, *args):
+    def shouldRecord(self, parts, *args):
         types = self.getSignalsToFilter()
         hasMouseEvent = DisplayFilter.instance.hasEventOfType(types, self.getGefViewer().getControl())
-        if len(self.widget.selectedEditParts()) == 1:
-            return hasMouseEvent 
-        else:
-            return len(self.getStateDescription(part, *args)) > 0 and (hasMouseEvent or ViewerEvent.shouldRecord(self, part, *args))
-
+        return hasMouseEvent and len(parts) > 0
 
     def implies(self, stateChangeOutput, stateChangeEvent, *args):
         currOutput = self.outputForScript(*args)
@@ -403,12 +528,15 @@ class ViewerSelectEvent(ViewerEvent):
 class DragHolder():
     draggedPart = None
     sourceEvent = None
+    sourceSwtEvent = None
     @classmethod
-    def reset(self):
-        self.draggedPart = None
-        self.sourceEvent = None
+    def reset(cls):
+        cls.draggedPart = None
+        cls.sourceEvent = None
+        cls.sourceSwtEvent = None
     
 class ViewerDragAndDropEvent(ViewerEvent):
+    allInstances = {}
     def connectRecord(self, method):
         class DDListener(swt.events.DragDetectListener):
             def dragDetected(lself, event):#@NoSelf
@@ -418,19 +546,35 @@ class ViewerDragAndDropEvent(ViewerEvent):
         self.addDropListener(method)
 
     def applyToDragged(self, event, method):
-        if len(self.widget.selectedEditParts()) > 0 and  self.shouldDrag(event):
+        if len(self.widget.selectedEditParts()) > 0 and self.shouldDrag(event):
             DragHolder.draggedPart = self.widget.selectedEditParts()[0].part()#self.getGefViewer().findObjectAt(p)
             DragHolder.sourceEvent = self
+            DragHolder.sourceSwtEvent = event
 
     def addDropListener(self, method):
         class MListener(swt.events.MouseAdapter):
             def mouseUp(lself, event):#@NoSelf
-                if DragHolder.sourceEvent is not None:
-                    storytext.guishared.catchAll(method, DragHolder.draggedPart, event.x, event.y, DragHolder.sourceEvent)
-                    DragHolder.reset()
+                storytext.guishared.catchAll(self.handleDrop, event, method)
 
+        self.allInstances.setdefault(self.__class__, []).append(self)
         rcpsimulator.swtsimulator.runOnUIThread(self.getGefViewer().getControl().addMouseListener, MListener())
 
+    def handleDrop(self, event, method):
+        if DragHolder.sourceEvent is not None and self.__class__ is DragHolder.sourceEvent.__class__:
+            for instance in self.allInstances.get(self.__class__, []):
+            # No guarantee MouseUp appers on same widget
+                dropX, dropY = instance.getDropPosition(event)
+                if instance.isValidDragAndDrop(DragHolder.sourceSwtEvent, event, dropX, dropY):
+                    method(DragHolder.draggedPart, dropX, dropY, instance, DragHolder.sourceEvent)
+                    break
+            DragHolder.reset()
+
+    def isValidDragAndDrop(self, dragEvent, dropEvent, dropX, dropY):
+        return dragEvent.widget is not dropEvent.widget or dragEvent.x != dropX or dragEvent.y != dropY
+
+    def getDropPosition(self, event):
+        return event.x, event.y
+
     def getStateDescription(self, part, *args):
         partDesc = self.storeObjectDescription(part)
         targetDesc = self.getTargetDescription(part, *args)
@@ -451,20 +595,26 @@ class ViewerDragAndDropEvent(ViewerEvent):
         types = self.getSignalsToFilter()
         return DisplayFilter.instance.hasEventOfType(types, self.getGefViewer().getControl())
 
-    def generate(self, description, *args):
-        editPart, xPos, yPos = self.parseDescription(description)
-        if editPart:
-            self.widget.drag(editPart, xPos, yPos)
-        else:
-            raise UseCaseScriptError, "Could not find any edit part in viewer matching description " + repr(description)
+    def generate(self, partAndPos):
+        self.widget.drag(*partAndPos)
         
-    def parseDescription(self, description):
+    def parseArguments(self, description):
         sourceDesc, dest = description.split(" to ", 1)
-        xDesc, yDesc = dest.split(":", 1)
-        displayLocation = rcpsimulator.swtsimulator.runOnUIThread(self.widget.getFigureCanvas().toDisplayLocation, int(xDesc), int(yDesc))
         editPart = self.findEditPart(self.widget.rootEditPart(), sourceDesc)
+        if not editPart:
+            raise UseCaseScriptError, "could not find any objects in viewer matching description " + repr(sourceDesc)
+        
+        displayLocation = self.parseDestination(dest)
         return editPart, displayLocation.x, displayLocation.y
     
+    def parseDestination(self, dest):
+        if ":" not in dest:
+            raise UseCaseScriptError, "drag destination must be in the form <xpos>:<ypos>"
+        xDesc, yDesc = dest.split(":", 1)
+        if not xDesc.isdigit() or not yDesc.isdigit():
+            raise UseCaseScriptError, "drag destination must have numeric values"
+        return rcpsimulator.swtsimulator.runOnUIThread(self.widget.getFigureCanvas().toDisplayLocation, int(xDesc), int(yDesc))
+    
     def implies(self, stateChangeOutput, stateChangeEvent, *args):
         return isinstance(stateChangeEvent, ViewerSelectEvent)
     
