@@ -3,7 +3,7 @@
     Hook application events directly into that for synchronisation."""
 
 import logging
-from storytext import applicationEvent
+import storytext.guishared
 from storytext.javaswttoolkit.simulator import DisplayFilter
 from org.eclipse.core.runtime.jobs import Job, JobChangeAdapter
 from threading import Lock
@@ -11,6 +11,7 @@ from threading import Lock
 class JobListener(JobChangeAdapter):
     # Add things from customwidgetevents here, if desired...
     systemJobNames = []
+    instance = None
     def __init__(self):
         self.jobNameToUse = None
         self.jobCount = 0
@@ -19,6 +20,9 @@ class JobListener(JobChangeAdapter):
         self.logger = logging.getLogger("Eclipse RCP jobs")
         
     def done(self, e):
+        storytext.guishared.catchAll(self.jobDone, e)
+        
+    def jobDone(self, e):
         jobName = e.getJob().getName().lower()        
         self.alterJobCount(-1)
         self.logger.debug("Completed " + ("system" if e.getJob().isSystem() else "non-system") + " job '" + jobName + "' jobs = " + repr(self.jobCount))    
@@ -29,18 +33,18 @@ class JobListener(JobChangeAdapter):
             self.setComplete()
 
     def setComplete(self):
-        applicationEvent("completion of " + self.jobNameToUse, category="jobs", delayLevel=self.getDelayLevel())
+        DisplayFilter.registerApplicationEvent("completion of " + self.jobNameToUse, category="jobs")
         self.jobNameToUse = None
 
-    def getDelayLevel(self):
-        return len(DisplayFilter.instance.eventsFromUser) if DisplayFilter.instance else 0
-
     def alterJobCount(self, value):
         self.jobCountLock.acquire()
         self.jobCount += value
         self.jobCountLock.release()
 
     def scheduled(self, e):
+        storytext.guishared.catchAll(self.jobScheduled, e)
+        
+    def jobScheduled(self, e):
         jobName = e.getJob().getName().lower()
         self.alterJobCount(1)
         self.logger.debug("Scheduled job '" + jobName + "' jobs = " + repr(self.jobCount))
@@ -57,4 +61,5 @@ class JobListener(JobChangeAdapter):
         
     @classmethod
     def enable(cls, *args):
-        Job.getJobManager().addJobChangeListener(cls(*args))
+        cls.instance = cls(*args)
+        Job.getJobManager().addJobChangeListener(cls.instance)
