@@ -20,6 +20,7 @@ from org.eclipse.swt import SWT
 from org.eclipse.ui.forms.widgets import ExpandableComposite
 from org.eclipse.ui.forms.events import ExpansionAdapter
 from org.eclipse.swtbot.swt.finder.widgets import AbstractSWTBotControl
+from java.lang import NullPointerException
 
 class WidgetAdapter(swtsimulator.WidgetAdapter):
     widgetViewIds = {}
@@ -93,9 +94,16 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
         self.setWidgetAdapter()
         swtsimulator.runOnUIThread(self.cacheAndMonitorViews)
         swtsimulator.WidgetMonitor.monitorAllWidgets(self, *args, **kw)
+
+    def getViews(self):
+        # Working around bug in SWTBot that crashes in some circumstances here...
+        try:
+            return self.bot.views()
+        except NullPointerException:
+            return []
         
     def cacheAndMonitorViews(self):
-        for swtbotView in self.bot.views():
+        for swtbotView in self.getViews():
             ref = swtbotView.getViewReference()
             if ref not in self.allViews:
                 self.allViews.add(ref)
@@ -107,15 +115,27 @@ class WidgetMonitor(swtsimulator.WidgetMonitor):
                 self.uiMap.monitorWidget(adapter)
                 self.monitorMenus(swtbotView)
                 self.addTitleChangedListener(swtbotView)
-
+                
+    def handleReplayFailure(self, errorText, *args):
+        if "MenuItem has already been disposed" in errorText: # View Menus can easily get regenerated, try to re-monitor them.
+            swtsimulator.runOnUIThread(self.monitorAllMenus)
+    
+    def monitorAllMenus(self):
+        for swtbotView in self.getViews():
+            self.uiMap.logger.debug("Menu item disposed - remonitoring menu in view " + swtbotView.getViewReference().getId())
+            self.monitorMenus(swtbotView)
+                
     def setWidgetAdapter(self):
         WidgetAdapter.setAdapterClass(WidgetAdapter)
     
+    def propertyChanged(self, propertyId, botView):
+        if propertyId == IWorkbenchPartConstants.PROP_PART_NAME or propertyId == IWorkbenchPartConstants.PROP_TITLE:
+            self.monitorMenus(botView)
+    
     def addTitleChangedListener(self, botView):
         class PropertyListener(IPropertyListener):
             def propertyChanged(lself, source, propertyId):#@NoSelf
-                if propertyId == IWorkbenchPartConstants.PROP_PART_NAME:
-                    self.monitorMenus(botView)
+                storytext.guishared.catchAll(self.propertyChanged, propertyId, botView)
 
         view = botView.getViewReference().getView(False)
         if view is not None:
@@ -155,14 +175,20 @@ class PartActivateEvent(storytext.guishared.GuiEvent):
         page = self.widget.getViewReference().getPage()
         swtsimulator.runOnUIThread(page.addPartListener, RecordListener())
 
-    def generate(self, argumentString):
+    def generate(self, *args):
+        # The idea is to just do this, but it seems to cause strange things to happen
+        #internally. So we do it outside SWTBot instead.
+        #self.widget.setFocus()
+        page = self.widget.getViewReference().getPage()
+        view = self.widget.getViewReference().getView(False)
+        swtsimulator.runOnUIThread(page.activate, view)
+        
+    def parseArguments(self, argumentString):
         # The idea is to just do this, but it seems to cause strange things to happen
         #internally. So we do it outside SWTBot instead.
         #self.widget.setFocus()
         if self.widget.getViewReference().getTitle() == argumentString:
-            page = self.widget.getViewReference().getPage()
-            view = self.widget.getViewReference().getView(False)
-            swtsimulator.runOnUIThread(page.activate, view)
+            return argumentString
         else:
             raise UseCaseScriptError, "Could not find View named " + repr(argumentString) + " to activate."
 
@@ -197,7 +223,17 @@ class PartActivateEvent(storytext.guishared.GuiEvent):
 class SWTBotExpandableComposite(AbstractSWTBotControl):
     def clickOnCenter(self):
         firstChild = self.widget.getChildren()[0]
+        display = self.widget.getDisplay()
+        # click(True) will move the mouse pointer
+        # Can't find any other way to do this.
+        # We can at least move it back again when we're done to avoid other bad effects
+        currPos = swtsimulator.runOnUIThread(display.getCursorLocation)
         SWTBotExpandableComposite(firstChild).click(True)
+        event = Event()
+        event.type = SWT.MouseMove
+        event.x = currPos.x
+        event.y = currPos.y
+        swtsimulator.runOnUIThread(display.post, event)
 
 class ExpandableCompositeEvent(swtsimulator.SelectEvent):
     def shouldRecord(self, *args):
