@@ -9,8 +9,7 @@ class ScriptEngine(storytext.guishared.ScriptEngine):
     eventTypes = [
         (swing.JFrame       , [ simulator.FrameCloseEvent, simulator.KeyPressForTestingEvent ]),
         (swing.JButton      , [ simulator.ButtonClickEvent ]),
-        (swing.JRadioButton , [ simulator.ClickEvent ]),
-        (swing.JCheckBox    , [ simulator.ClickEvent ]),
+        (swing.JToggleButton, [ simulator.ClickEvent ]),
         (swing.JMenuItem    , [ simulator.MenuSelectEvent ]),
         (swing.JTabbedPane  , [ simulator.TabSelectEvent, simulator.TabPopupActivateEvent]),
         (swing.JDialog      , [ simulator.FrameCloseEvent ]),
@@ -50,7 +49,7 @@ class ScriptEngine(storytext.guishared.ScriptEngine):
     def cleanup(self, interface):
         for frame in Frame.getFrames():
             frame.setVisible(False) # don't leave the window up, looks weird
-        self.replaceAutoRecordingForUsecase(interface)
+        self.replaceAutoRecordingForUsecase(interface, exitHook=True)
     
     def checkType(self, widget):
         # Headers are connected to the table to use any identification that is there
@@ -103,7 +102,7 @@ class UseCaseReplayer(storytext.guishared.ThreadedUseCaseReplayer):
         inWindow = isinstance(widget, swing.JComponent) and widget.getTopLevelAncestor() is not None and \
                    widget.getTopLevelAncestor() in self.uiMap.windows
         isWindow = isinstance(widget, (swing.JFrame, swing.JDialog))
-        appEventButton = hasattr(widget, "getText") and str(widget.getText()).startswith("ApplicationEvent")
+        appEventButton = hasattr(widget, "getText") and unicode(widget.getText()).startswith("ApplicationEvent")
         popupMenu = isinstance(widget, swing.JPopupMenu) and not util.belongsMenubar(widget.getInvoker())
         if self.uiMap and (self.isActive() or self.recorder.isActive()):
             if isWindow:
@@ -120,8 +119,9 @@ class UseCaseReplayer(storytext.guishared.ThreadedUseCaseReplayer):
 
     def setAppeared(self, widget):
         self.appearedWidgets.add(widget)
-        for child in widget.getComponents():
-            self.setAppeared(child)
+        if hasattr(widget, "getComponents"):
+            for child in widget.getComponents():
+                self.setAppeared(child)
 
     def describe(self):
         util.runOnEventDispatchThread(self.describer.describeWithUpdates)
