@@ -1,5 +1,5 @@
 
-import storytext.guishared, util, types, logging, sys
+import storytext.guishared, util, types, logging, sys, os
 from storytext.definitions import UseCaseScriptError
 from storytext.gridformatter import GridFormatter
 from org.eclipse import swt
@@ -7,13 +7,16 @@ from browserhtmlparser import BrowserHtmlParser
 from java.util import Date
 from java.io import File, FilenameFilter
 from org.eclipse.jface.resource import ImageDescriptor
+from array import array
+from ordereddict import OrderedDict
 
 class Describer(storytext.guishared.Describer):
     styleNames = [ (swt.widgets.CoolItem, []),
                    (swt.widgets.Item    , [ "SEPARATOR", "DROP_DOWN", "CHECK", "CASCADE", "RADIO" ]),
                    (swt.widgets.Button  , [ "CHECK", "RADIO", "TOGGLE", "ARROW", "UP", "DOWN" ]),
                    (swt.widgets.DateTime, [ "DATE", "TIME", "CALENDAR", "SHORT" ]),
-                   (swt.widgets.Combo   , [ "READ_ONLY", "SIMPLE" ]), 
+                   (swt.widgets.Combo   , [ "READ_ONLY", "SIMPLE" ]),
+                   (swt.custom.CCombo   , [ "READ_ONLY", "FLAT", "BORDER" ]), 
                    (swt.widgets.Text    , [ "PASSWORD", "SEARCH", "READ_ONLY" ]) ]
     ignoreWidgets = [ types.NoneType ]
     # DateTime children are an implementation detail
@@ -24,7 +27,8 @@ class Describer(storytext.guishared.Describer):
                      swt.widgets.CoolBar, swt.widgets.ToolBar, swt.widgets.Label, swt.custom.CLabel,
                      swt.widgets.Combo, swt.widgets.ExpandBar, swt.widgets.Text, swt.widgets.List,
                      swt.widgets.Tree, swt.widgets.DateTime, swt.widgets.TabFolder, swt.widgets.Table, 
-                     swt.custom.CTabFolder, swt.widgets.Canvas, swt.browser.Browser, swt.widgets.Composite ]
+                     swt.custom.CTabFolder, swt.widgets.Canvas, swt.browser.Browser, swt.custom.CCombo,
+                     swt.widgets.Composite ]
     childrenMethodName = "getChildren"
     visibleMethodName = "getVisible"
     def __init__(self):
@@ -34,7 +38,7 @@ class Describer(storytext.guishared.Describer):
         self.widgetsDescribed = set()
         self.browserStates = {}
         self.clipboardText = None
-        self.imageDescriptorToNames = {}
+        self.storedImages = {}
         self.imageToName = {}
         self.handleImages()
         storytext.guishared.Describer.__init__(self)
@@ -54,16 +58,17 @@ class Describer(storytext.guishared.Describer):
     def addFilters(self, display):
         class ShowListener(swt.widgets.Listener):
             def handleEvent(listenerSelf, e):
-                self.setWidgetShown(e.widget)
+                storytext.guishared.catchAll(self.setWidgetShown, e.widget)
 
         class PaintListener(swt.widgets.Listener):
             def handleEvent(listenerSelf, e):
-                self.setWidgetPainted(e.widget)
+                storytext.guishared.catchAll(self.setWidgetPainted, e.widget)
 
         display.addFilter(swt.SWT.Show, ShowListener())
         display.addFilter(swt.SWT.Paint, PaintListener())
 
-    def describeWithUpdates(self, shell):
+    def describeWithUpdates(self, shellMethod):
+        shell = shellMethod()
         if shell in self.windows:
             stateChanges = self.findStateChanges(shell)
             stateChangeWidgets = [ widget for widget, old, new in stateChanges ]
@@ -78,7 +83,10 @@ class Describer(storytext.guishared.Describer):
         self.widgetsAppeared = []
         
     def shouldCheckForUpdates(self, widget, shell):
-        return not isinstance(widget, swt.widgets.Menu) or (not widget.isDisposed() and widget.getShell() == shell)
+        return not isinstance(widget, self.getMenuClasses()) or (not widget.isDisposed() and widget.getShell() == shell)
+    
+    def getMenuClasses(self):
+        return swt.widgets.Menu
 
     def widgetShowing(self, widget):
         return not widget.isDisposed() and util.isVisible(widget)
@@ -173,7 +181,7 @@ class Describer(storytext.guishared.Describer):
         return self.getMenuDescription(menu, indent=2)
     
     def getMenuBarDescription(self, menubar):
-        if menubar:
+        if menubar and "Menu" not in self.excludeClassNames:
             return "Menu Bar:\n" + self.getMenuDescription(menubar, storeStatesForSubMenus=True)
         else:
             return ""
@@ -224,17 +232,41 @@ class Describer(storytext.guishared.Describer):
             name += " " + self.imageCounter.getId(image)
         return name
     
+    def getPixels(self, data):
+        pixels = array('i', (0, ) * data.width * data.height)
+        data.getPixels(0, 0, data.width * data.height, pixels, 0)
+        return pixels
+
+    def imageDataMatches(self, data, data2, hasExcessData):
+        if hasExcessData:
+            return self.getPixels(data) == self.getPixels(data2)
+        else:
+            return data.data == data2.data
+
     def getImageName(self, image):
         name = self.imageToName.get(image)
         if name is not None:
             return name
-        for img in sorted(self.imageDescriptorToNames, key=lambda x: self.imageDescriptorToNames[x]):
-            newImage = img.createImage()
-            if newImage.getImageData().data == image.getImageData().data:
-                name = self.imageDescriptorToNames.get(img)                
-                self.imageToName[image] = name                
-                break
-        return name
+
+        data = image.getImageData()
+        hasExcessData = data.width * data.depth / 8 < data.bytesPerLine
+        imageDict = self.storedImages.get((data.width, data.height), {})
+        for name, imgData in imageDict.items():
+            if self.imageDataMatches(data, imgData, hasExcessData):
+                baseName = os.path.basename(name)
+                self.imageToName[image] = baseName             
+                return baseName
+       
+    def storeImageData(self, url):
+        imgDesc = ImageDescriptor.createFromURL(url)
+        name = url.getFile()
+        if imgDesc is not None:
+            newImage = imgDesc.createImage()
+            data = newImage.getImageData()
+            imageDict = self.storedImages.setdefault((data.width, data.height), OrderedDict())
+            if name not in imageDict:
+                imageDict[name] = data
+            newImage.dispose()
 
     def getCanvasDescription(self, widget):
         return "Canvas " + self.canvasCounter.getId(widget)
@@ -290,7 +322,8 @@ class Describer(storytext.guishared.Describer):
     def getControlDecorationDescription(self, item):
         deco = self.getControlDecoration(item)
         if deco:
-            imgDesc = self.getControlDecorationImageDescription(deco.getImage())
+            image = deco.getImage()
+            imgDesc = self.getImageDescription(deco.getImage()) if image is not None else ""
         if deco and self.decorationVisible(deco): 
             text = "Decoration " + imgDesc
             desc = deco.getDescriptionText()
@@ -308,13 +341,6 @@ class Describer(storytext.guishared.Describer):
         method.setAccessible(True)
         return method.invoke(deco, *args)
 
-    def getControlDecorationImageDescription(self, image):
-        # Hard coded. It gets indeterministic results
-        if self.imageDescriptionType == "name":
-            return "Unknown Image"
-        else:
-            return self.getDefaultImageDescription(image)
-        
     def getPropertyElements(self, item, selected=False):
         elements = []
         decoText = self.getControlDecorationDescription(item)
@@ -330,6 +356,8 @@ class Describer(storytext.guishared.Describer):
         if selected:
             elements.append("selected")
         elements.append(self.getContextMenuReference(item))
+        if hasattr(item, "getItemCount") and hasattr(item, "getExpanded") and item.getItemCount() > 0 and not item.getExpanded():
+            elements.append("+")
         return elements
 
     def getLabelState(self, label):
@@ -389,7 +417,8 @@ class Describer(storytext.guishared.Describer):
     def getBrowserState(self, widget):
         url = util.getRealUrl(widget)
         # Ignore non-breaking spaces, they are invisible anyway
-        rawText = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ")
+        # Webkit returns them in invalid format without the semicolons... handle that too.
+        rawText = widget.getText().replace(u"\xa0", " ").replace("&nbsp;", " ").replace("&nbsp", " ")
         text = storytext.guishared.encodeToLocale(rawText)
         return url or BrowserHtmlParser().parse(text)
         
@@ -431,6 +460,9 @@ class Describer(storytext.guishared.Describer):
     def getComboDescription(self, widget):
         return self.getTextDescription(widget)
 
+    getCComboDescription = getComboDescription
+    getCComboState = getComboState
+
     def getTreeDescription(self, widget):
         return self.getAndStoreState(widget)
 
@@ -469,7 +501,7 @@ class Describer(storytext.guishared.Describer):
         return text
 
     def getContextMenuReference(self, widget):
-        if not isinstance(widget, swt.widgets.MenuItem) and hasattr(widget, "getMenu") and widget.getMenu():
+        if "Menu" not in self.excludeClassNames and not isinstance(widget, swt.widgets.MenuItem) and hasattr(widget, "getMenu") and widget.getMenu():
             return "Context Menu " + self.contextMenuCounter.getId(widget.getMenu())
         else:
             return ""
@@ -492,12 +524,23 @@ class Describer(storytext.guishared.Describer):
         columns = widget.getColumns()
         columnCount = len(columns)
         text = self.combineElements([ "Table" ] + self.getPropertyElements(widget)) + " :\n"
-        rows = self.getAllItemDescriptions(widget, indent=0, 
+        rows = self.getAllTableItemDescriptions(widget, indent=0, 
                                            selection=widget.getSelection(),
                                            columnCount=columnCount)
-        headerRow = [ c.getText() for c in columns ]
+        headerRow = [ c.getText() for c in columns if c.getWidth() > 0] # Don't show hidden columns
         return text + self.formatTable(headerRow, rows, columnCount)
 
+    def getAllTableItemDescriptions(self, widget, indent=0,
+                               prefix="", selection=[], columnCount=0, **kw):
+        descs = []
+        for item in widget.getItems():
+            currPrefix = prefix + " " * indent * 2
+            selected = item in selection
+            if columnCount:
+                row = [ self.getItemColumnDescription(item, i, currPrefix, selected) for i in range(columnCount) if widget.getColumn(i).getWidth() > 0]
+                descs.append(row)
+        return descs
+
     def getTabFolderDescription(self, widget):
         state = self.getState(widget)
         self.widgetsWithState[widget] = state
@@ -580,10 +623,11 @@ class Describer(storytext.guishared.Describer):
         
     def formatContextMenuDescriptions(self):
         text = ""
-        for contextMenu, menuId in self.contextMenuCounter.getWidgetsForDescribe():
-            menuDesc = self.getMenuDescription(contextMenu)
-            text += "\n\nContext Menu " + str(menuId) + ":\n" + menuDesc
-            self.widgetsWithState[contextMenu] = self.getMenuState(contextMenu) 
+        if "Menu" not in self.excludeClassNames:
+            for contextMenu, menuId in self.contextMenuCounter.getWidgetsForDescribe():
+                menuDesc = self.getMenuDescription(contextMenu)
+                text += "\n\nContext Menu " + str(menuId) + ":\n" + menuDesc
+                self.widgetsWithState[contextMenu] = self.getMenuState(contextMenu) 
         return text
 
     def getHorizontalSpan(self, widget, columns):
@@ -627,16 +671,9 @@ class Describer(storytext.guishared.Describer):
 
         return d.listFiles(Filter())
     
-    def makeImageDescriptor(self, url):
-        imgDesc = ImageDescriptor.createFromURL(url)
-        name = url.getFile().split("/")[-1]
-        if imgDesc is not None:
-            #print "FILE", url.getFile()
-            self.imageDescriptorToNames[imgDesc] = name
-
     def findFiles(self, pathAsFile):
         if pathAsFile.isFile() and self.isImageType(pathAsFile.getName()):
-            self.makeImageDescriptor(pathAsFile.toURI().toURL())
+            self.storeImageData(pathAsFile.toURI().toURL())
         elif pathAsFile.isDirectory():
             for f in pathAsFile.listFiles():
                 if f is not None:
@@ -648,3 +685,12 @@ class Describer(storytext.guishared.Describer):
     def checkWindow(self, window):
         # Don't describe tooltips
         return (window.getStyle() & swt.SWT.TOOL) == 0
+    
+    def getStateChangeDescription(self, widget, oldState, state):
+        if isinstance(widget, (swt.widgets.Menu, swt.widgets.ToolBar)):
+            old = oldState.split("\n")
+            new = state.split("\n")
+            if len(old) == len(new):
+                return self.getDiffedDescription(widget, old, new)
+        return storytext.guishared.Describer.getStateChangeDescription(self, widget, oldState, state)
+
