@@ -8,9 +8,35 @@ def getRealUrl(browser):
     url = browser.getUrl()
     return url if url != "about:blank" else ""
 
+class TextLabelFinder(storytext.guishared.TextLabelFinder):
+    def getLabelClass(self):
+        return swt.widgets.Label, swt.custom.CLabel
+
+    def getChildren(self, widget):
+        return widget.getChildren()
+    
+    def getEarliestRelevantIndex(self, widgetPos, children, parent):
+        if not isinstance(parent.getLayout(), swt.layout.GridLayout):
+            return 0
+        
+        numColumns = parent.getLayout().numColumns
+        if numColumns == 1: # If there's only one column, don't worry about it...
+            return 0
+        
+        currIndex = 0
+        rows = {}
+        for ix, child in enumerate(children):
+            span = min(child.getLayoutData().horizontalSpan, numColumns)
+            row = currIndex / numColumns
+            rows.setdefault(row, []).append(ix)
+            if ix == widgetPos:
+                return rows[row][0]
+            else:
+                currIndex += span
+
 ignoreLabels = []
 def getTextLabel(widget):
-    return storytext.guishared.getTextLabel(widget, "getChildren", swt.widgets.Label, ignoreLabels)
+    return TextLabelFinder(widget, ignoreLabels).find()
 
 def getInt(intOrMethod):
     return intOrMethod if isinstance(intOrMethod, int) else intOrMethod()
