@@ -8,16 +8,16 @@ from copy import copy
 waitRegexp = re.compile(waitCommandName + ".*")
 
 class ReplayScript:
-    def __init__(self, scriptName):
+    def __init__(self, scriptName, ignoreComments=False):
         self.commands = []
         self.exitObservers = []
         self.pointer = 0
         self.name = scriptName
         if not os.path.isfile(scriptName):
             raise UseCaseScriptError, "Cannot replay script " + repr(scriptName) + ", no such file or directory."
-        for line in open(scriptName).xreadlines():
+        for line in open(scriptName):
             line = line.strip()
-            if line != "" and line[0] != "#":
+            if not ignoreComments or (line != "" and line[0] != "#"):
                 self.commands.append(line)
 
     def addExitObserver(self, observer):
@@ -35,8 +35,16 @@ class ReplayScript:
             name = name.replace("$", arg, 1)
         return name
 
+    def transformToRegexp(self, text):
+        for char in "^[]{}*?|+()":
+            text = text.replace(char, "\\" + char)
+        # handle numbered variables
+        text = re.sub("\$([0-9]+)", "(?P<var\\1>.*)", text)
+        # handle unnumbered variables, and make sure we don't match anything longer
+        return text.replace("$", "(.*)") + "$"
+    
     def getRegexp(self, command):
-        return re.compile(command.replace("$", "(.*)")) if command else None
+        return re.compile(self.transformToRegexp(command)) if command else None
 
     def hasTerminated(self):
         return self.pointer >= len(self.commands)
@@ -62,35 +70,105 @@ class ReplayScript:
                 self.pointer += 1
                 return self.replaceArgs(nextCommand, args)
 
+
+    def getArgument(self, nextCommand, args):
+        pos = nextCommand.find("$")
+        if pos + 1 < len(nextCommand):
+            followingChar = nextCommand[pos + 1]
+            if followingChar.isdigit():
+                index = int(followingChar) - 1
+                if index < len(args):
+                    return args[index]
+        currArg = args.pop(0)
+        args.append(currArg) # cycle through them...
+        return currArg
+
     def replaceArgs(self, nextCommand, args):
         if args:
             while "$" in nextCommand:
-                currArg = args.pop(0)
-                args.append(currArg) # cycle through them...
-                nextCommand = nextCommand.replace("$", currArg, 1)
+                currArg = self.getArgument(nextCommand, args)
+                nextCommand = re.sub("\$[0-9]*", currArg, nextCommand, 1)
         return nextCommand
             
-    def getCommandsSoFar(self):
-        return self.commands[:self.pointer - 1] if self.pointer else []
+    def getCommandsSoFar(self, args):
+        rawCommands = self.commands[:self.pointer - 1] if self.pointer else []
+        return [ self.replaceArgs(cmd, args) for cmd in rawCommands]
+
+    @staticmethod
+    def isComment(command):
+        return len(command) == 0 or command.startswith("#")
+
+    def extractAllComments(self):
+        comments = []
+        while self.pointer < len(self.commands):
+            nextCommand = self.commands[self.pointer]
+            if self.isComment(nextCommand):
+                comments.append(nextCommand)
+                self.pointer += 1
+            else:
+                break
+        return comments
 
     def getCommands(self, args):
+        commands = self.extractAllComments()
         command = self.getCommand(args)
-        if not command:
-            return []
+        if command is None:
+            return commands
+        
+        commands.append(command)
+        pointerWithCommand = self.pointer
+        commentsAfter = self.extractAllComments()
 
         # Process application events together with the previous command so the log comes out sensibly...
         waitCommand = self.getCommand(args, [ waitRegexp ])
         if waitCommand:
-            return [ command, waitCommand ]
+            commands += commentsAfter
+            commands.append(waitCommand)
         else:
-            return [ command ]
+            self.pointer = pointerWithCommand
+        return commands
         
+        
+class ShortcutManager:
+    def __init__(self):
+        self.shortcuts = []
+        
+    def add(self, shortcut):
+        self.shortcuts.append((shortcut.getShortcutRegexp(), shortcut))
+
+    def getShortcuts(self):
+        # Drop the trailing $ from the pattern
+        return sorted(((r.pattern[:-1], shortcut) for r, shortcut in self.shortcuts))
+    
+    def getRegexps(self):
+        return [ r for r, _ in self.shortcuts ]
+    
+    def findShortcut(self, command):
+        bestShortcut, bestArgs = None, []
+        for regex, shortcut in self.shortcuts:
+            match = regex.match(command)
+            if match:
+                args = list(match.groups())
+                if bestShortcut is None or self.isBetterShortcut(args, bestArgs):
+                    bestShortcut, bestArgs = shortcut, args
+        return bestShortcut, bestArgs
+
+    def isBetterShortcut(self, args1, args2):
+        if len(args1) != len(args2):
+            return len(args1) > len(args2)
+        
+        # If we have the same arguments, choose the shortcut with least text in arguments
+        # which implies more text in the given name
+        argLength1 = sum(map(len, args1))
+        argLength2 = sum(map(len, args2))
+        return argLength1 < argLength2
+    
     
 class UseCaseReplayer:
     def __init__(self, timeout=60):
         self.logger = logging.getLogger("storytext replay log")
         self.scripts = []
-        self.shortcuts = []
+        self.shortcutManager = ShortcutManager()
         self.events = {}
         self.waitingForEvents = []
         self.applicationEventNames = set()
@@ -110,10 +188,10 @@ class UseCaseReplayer:
         return len(self.scripts) > 0
 
     def registerShortcut(self, shortcut):
-        self.shortcuts.append((shortcut.getShortcutRegexp(), shortcut))
+        self.shortcutManager.add(shortcut)
 
     def getShortcuts(self):
-        return sorted(((r.pattern, shortcut) for r, shortcut in self.shortcuts))
+        return self.shortcutManager.getShortcuts()
     
     def addEvent(self, event):
         self.events.setdefault(event.name, []).append(event)
@@ -128,10 +206,10 @@ class UseCaseReplayer:
             self.runScript(script, enableReading=True)
 
     def runScript(self, script, enableReading):
-        if self.shortcuts:
-            scriptCommand = script.getCommand(matching=[ r for r, _ in self.shortcuts ])
+        if self.shortcutManager.shortcuts:
+            scriptCommand = script.getCommand(matching=self.shortcutManager.getRegexps())
             if scriptCommand:
-                newScript, args = self.findShortcut(scriptCommand)
+                newScript, args = self.shortcutManager.findShortcut(scriptCommand)
                 return self.addScript(newScript, args, enableReading)
         
         if enableReading and self.processInitialWait(script):
@@ -141,6 +219,7 @@ class UseCaseReplayer:
         waitCommand = script.getCommand(matching=[ waitRegexp ])
         if waitCommand:
             self.logger.debug("Initial " + repr(waitCommand) + ", not starting replayer")
+            self.handleComment(None) # Keep the comments in the right order...
             return self.processWait(self.getArgument(waitCommand, waitCommandName))
         else:
             return True
@@ -249,12 +328,8 @@ class UseCaseReplayer:
             self.write(self.eventHappenedMessage)
             self.eventHappenedMessage = ""
 
-    def findShortcut(self, command):
-        for regex, shortcut in self.shortcuts:
-            match = regex.match(command)
-            if match:
-                return shortcut, list(match.groups())
-        return None, None
+    def handleComment(self, comment):
+        pass # Can't do anything here, don't know about the recorder...
 
     def runNextCommand(self, **kw):
         if self.timeDelayNextCommand:
@@ -264,8 +339,13 @@ class UseCaseReplayer:
         commands = self.getCommands()
         if len(commands) == 0:
             return False, False
+        
         for command in commands:
-            script, arguments = self.findShortcut(command)
+            if ReplayScript.isComment(command):
+                self.handleComment(command)
+                continue
+            
+            script, arguments = self.shortcutManager.findShortcut(command)
             if script:
                 self.logger.debug("Found shortcut '" + script.getShortcutName() + "' adding to list of script")
                 if commands[-1].startswith(waitCommandName):
@@ -273,6 +353,10 @@ class UseCaseReplayer:
                     script.commands.append(commands[-1])
                 self.addScript(script, arguments)
                 return self.runNextCommand(**kw)
+            else:
+                #  Add a delimiter to show that we've replayed something else
+                # Helps the recorder know what order to put things in
+                self.handleComment(None)
             if command.startswith(waitCommandName):
                 eventName = self.getArgument(command, waitCommandName)
                 if self.processWait(eventName):
@@ -293,23 +377,29 @@ class UseCaseReplayer:
             # This isn't worth crashing over!
             pass
 
+    def _parseAndProcess(self, command, **kw):
+        self.describeAppEventsHappened()
+        commandName, argumentString = self.parseCommand(command)
+        self.logger.debug("About to perform " + repr(commandName) + " with arguments " + repr(argumentString))
+        self.processCommand(commandName, argumentString)
+
     def parseAndProcess(self, command, **kw):
         try:
-            self.describeAppEventsHappened()
-            commandName, argumentString = self.parseCommand(command)
-            self.logger.debug("About to perform " + repr(commandName) + " with arguments " + repr(argumentString))
-            self.processCommand(commandName, argumentString)
+            self._parseAndProcess(command, **kw)
         except UseCaseScriptError:
             # We don't terminate scripts if they contain errors
             type, value, _ = sys.exc_info()
-            self.write("ERROR: " + str(value))
+            self.write("ERROR: Could not simulate command " + repr(command) + " - " + str(value))
+            
+    def describeEvent(self, commandName, argumentString):
+        self.write("")
+        self.write("'" + commandName + "' event created with arguments '" + argumentString + "'")
 
     def processCommand(self, commandName, argumentString):
         if commandName == signalCommandName:
             self.processSignalCommand(argumentString)
         else:
-            self.write("")
-            self.write("'" + commandName + "' event created with arguments '" + argumentString + "'")
+            self.describeEvent(commandName, argumentString)
             possibleEvents = self.events[commandName]
             # We may have several plausible events with this name,
             # but some of them won't work because widgets are disabled, invisible etc
@@ -332,7 +422,7 @@ class UseCaseReplayer:
         return commandName, argumentString
 
     def getParseError(self, scriptCommand):
-        return "Could not parse script command '" + scriptCommand + "'"
+        return "could not parse the given text."
 
     def getArgument(self, scriptCommand, commandName):
         return scriptCommand.replace(commandName, "").strip()
@@ -346,6 +436,20 @@ class UseCaseReplayer:
             if command.startswith(eventName) and len(eventName) > len(longestEventName):
                 longestEventName = eventName
         return longestEventName            
+
+    def startTimer(self, timer):
+        self.appEventTimer = timer
+        self.appEventTimer.start()
+
+    def setAppEventTimer(self):
+        # Break the timer up into 5 sub-timers
+        # The point is to prevent timing out too early if the process gets suspended
+        subTimerCount = 5 # whatever
+        subTimerTimeout = float(self.appEventTimeout) / subTimerCount
+        timer = Timer(subTimerTimeout, self.timeoutApplicationEvents)
+        for _ in range(subTimerCount - 1):
+            timer = Timer(subTimerTimeout, self.startTimer, [ timer ])
+        self.startTimer(timer)
     
     def processWait(self, applicationEventStr):
         eventsToWaitFor = applicationEventStr.split(", ")
@@ -355,8 +459,7 @@ class UseCaseReplayer:
         self.waitingForEvents = allEventsToWaitFor
         complete = self.waitingCompleted()
         if not complete:
-            self.appEventTimer = Timer(self.appEventTimeout, self.timeoutApplicationEvents)
-            self.appEventTimer.start()
+            self.setAppEventTimer()
         return complete
     
     def processSignalCommand(self, signalArg):
