@@ -49,7 +49,7 @@ class ScriptEngine:
         for fileName in sorted(os.listdir(self.storytextHome)):
             if fileName.endswith(".shortcut"):
                 fullPath = os.path.join(self.storytextHome, fileName)
-                shortcuts.append(replayer.ReplayScript(fullPath))
+                shortcuts.append(replayer.ReplayScript(fullPath, ignoreComments=True))
         return shortcuts
 
     def createReplayer(self, **kw):
@@ -69,9 +69,9 @@ class ScriptEngine:
         if self.replayerActive():
             self.replayer.applicationEventRename(oldName, newName)
 
-    def applicationEventDelay(self, name):
+    def applicationEventDelay(self, name, **kw):
         if self.recorderActive():
-            self.recorder.applicationEventDelay(name)
+            self.recorder.applicationEventDelay(name, **kw)
 
     def run(self, options, args):
         if len(args) == 0:
